/******** header files ********/
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>

/****** local header files ******/
#include "gfx_functions.h"
#include "SFont.h"

#ifndef pi
#define pi 3.141592653589793238462643383279502884197169399375105820974944
#endif //ifndef pi
/******** functions ********/
void initialize_screen(void);
void handle_events(void);

/****************** global variables ******************/
int mousebutton_state, mousex, mousey;
SDL_Surface *mask, *screenbuffer;
SFont_Font *font_1;
int numballs = 10;

class ballclass{
	public:
		int x, y, xi, yi;
		float ang, angi, scale;
		SDL_Surface *image;

		ballclass(){
			image = (SDL_Surface *)NULL;
			x = y = xi = yi = 0;
			ang = 0.0;
			angi = 0;
		}

};

ballclass ball[10];

int main(void){
	int n;

	atexit(&gfx_free_memory);

	initialize_screen();
	screenbuffer = SDL_CreateRGBSurface(SDL_SWSURFACE, x_resolution, y_resolution, 32, RMASK, GMASK, BMASK, AMASK);
	if(screenbuffer == (SDL_Surface *)NULL) exit(1);

	mask = gfx_load_png("origami_mask.png");
	if(mask == (SDL_Surface *)NULL) exit(1);
	else gfx_add_freepointer(mask);

	for(n = 0; n < numballs; n++){
		ball[n].image = mask;
		ball[n].x = rand() % x_resolution;
		ball[n].y = rand() % y_resolution;
		ball[n].xi = rand() % 3 + 4;
		ball[n].yi = 0;
		ball[n].scale = 1.0 + (float)n / numballs;
		ball[n].ang = (float)(rand() % 314) / 50.0;
		ball[n].angi = 0.15;
		if(rand() % 2){
			ball[n].angi *= -1.0;
			ball[n].xi *= -1;
		}
	}

	/*** Draw a background image on the screen ***/
	gfx_solid_box(screenbuffer, 0, 0, x_resolution - 1, y_resolution - 1, 0, 0, 0, 255);
	
	gfx_blit(screen_image, 0, 0, screenbuffer);
	SDL_UpdateRect(screen_image, 0, 0, 0, 0);
	/*** Now do some animation ***/
	do{
		for(n = 0; n < numballs; n++){
			ball[n].x  += ball[n].xi;
			ball[n].y  += ball[n].yi;
			ball[n].ang += ball[n].angi;
			if(ball[n].x > x_resolution){
				ball[n].xi = -abs(ball[n].xi);
				ball[n].angi = -fabs(ball[n].angi);
			}

			if(ball[n].x < 0){
				ball[n].xi = abs(ball[n].xi);
				ball[n].angi = fabs(ball[n].angi);
			}

			ball[n].yi++;
			if(ball[n].y > y_resolution) ball[n].yi = 1 - abs(ball[n].yi);
			if(ball[n].y < 0) ball[n].yi = abs(ball[n].yi);
			if(ball[n].ang > 2 * pi) ball[n].ang -= 2 * pi;
			if(ball[n].ang < 0) ball[n].ang += 2 * pi;

		}
		SDL_FillRect(screen_image, NULL, SDL_MapRGBA(screen_image->format, 0, 0, 0, 0));
		for(n = 0; n < numballs; n++){
			gfx_rotblit(screen_image, ball[n].x, ball[n].y, ball[n].image->w >> 1, ball[n].image->h >> 1, ball[n].ang, ball[n].scale, ball[n].image);
		}
		SDL_UpdateRect(screen_image, 0, 0, 0, 0);
		handle_events();
	}while(mousebutton_state != 1);
	SDL_FreeSurface(screenbuffer);
	return 0;
}

void initialize_screen(void){
	/** initialize the graphics **/
	if(SDL_Init(SDL_INIT_VIDEO) < 0){
		/** can't initialize SDL **/
		fprintf(stderr, "Unable to initialize SDL\n");
		exit(1);
	}

	atexit(SDL_Quit);
	if(gfx_init(640, 480, 32, 0 & SDL_FULLSCREEN)){
		fprintf(stderr, "Unable to initialize graphics\n");
		exit(1);
	}
}

void handle_events(void){
        SDL_Event event;
        while(SDL_PollEvent(&event)){
                switch(event.type){
                        case SDL_MOUSEBUTTONUP:
                                if(event.button.button == SDL_BUTTON_LEFT) mousebutton_state &= 254; 
                                if(event.button.button == SDL_BUTTON_RIGHT) mousebutton_state &= 253; 
                                if(event.button.button == SDL_BUTTON_MIDDLE) mousebutton_state &= 251; 
                                break;  
                        case SDL_MOUSEBUTTONDOWN:
                                if(event.button.button == SDL_BUTTON_LEFT) mousebutton_state |= 1;
                                if(event.button.button == SDL_BUTTON_RIGHT) mousebutton_state |= 2;
                                if(event.button.button == SDL_BUTTON_MIDDLE) mousebutton_state |= 4;
                                break;  
                        case SDL_MOUSEMOTION:
                                mousex = event.motion.x;
                                mousey = event.motion.y;
                                break;  
                        default:
                                break;  
                }
        }
}
